# Bloque

Cada `Block` atiende a un único objetivo. Contendrá su propia pestaña, formulario y lógica para ejecutar su funcionalidad con las opciones marcadas por el administrador.

Veamos por ejemplo el bloque `XML-RPC`:

![Alt text](images/dashboard-xmlrpc.png)

Contiene un único campo de formulario que permite desactivar este protocolo susceptible de ataques.

## Clase

Siempre hereda de la clase `Block`:

```php
class XmlRpcBlock extends Block {
  // Contenido...
}
```

## Constructor

```php
public function __construct() {
  parent::__construct(
    __( 'XML-RPC', 'flat101-security' ),
    'xmlrpc',
    array(
      'disabled' => true,
    )
  );
}
```

Al constructor de la clase base le pasamos 3 campos:

-   `$title` Título del bloque. Será el texto visible de la pestaña.

-   `$unique_key` Identifica el bloque, debe ser único y se usará como prefijo para todos los campos (settings) que contenga.

-   `$default_settings` Array clave-valor con todas las opciones que podrá configurar el administrador con su valor por defecto. Por regla general cada setting corresponde con un campo del formulario.

## Método _get_description()_

![Alt text](images/block-description.png)

Método opcional. Sirve para añadir texto descriptivo en la cabecera del bloque.

Si se requiere HTML se intentará que no forme parte de la traducción como en este caso:

```php
// PHPDoc (inherited).
// phpcs:ignore Squiz.Commenting.FunctionComment.Missing
public function get_description() {

  $text = esc_html_x( 'this link', 'text for a HTML link', 'flat101-security' );
  $link = sprintf( '<a href="http://scripting.com/code/xmlrpcbrowserclient/index.html" target="_blank">%s</a>', $text );

  /* translators: %s: is a HTML link */
  return sprintf( esc_html__( 'You can use %s to check if XML-RPC is active on your site.', 'flat101-security' ), $link );
}
```

## Método get_fields()

Devuelve un array de campos del formulario. Para ello empleamos [Carbon Fields](https://docs.carbonfields.net/learn/fields/usage.html).

```php
// PHPDoc (inherited).
// phpcs:ignore Squiz.Commenting.FunctionComment.Missing
public function get_fields() {
  return array(

    Field::make( 'checkbox', $this->get_option_name( 'disabled' ), __( 'Disabled', 'flat101-security' ) )
    ->set_default_value( $this->default_settings['disabled'] )
    ->set_help_text(
      __( 'Avoid DDoS and Brute Force attacks by disabling XML-RPC.', 'flat101-security' )
    ),

  );
}
```

## Método run()

Ejecuta la lógica del bloque basándose en los settings (que ya estarán debidamente inicializados cuando se llame a esta función).

```php
// PHPDoc (inherited).
// phpcs:ignore Squiz.Commenting.FunctionComment.Missing
public function run() {
  if ( $this->settings['disabled'] ) {
    add_filter( 'xmlrpc_enabled', '__return_false' );
  }
}
```

> Esta función se ejecuta dentro de la acción [init](https://developer.wordpress.org/reference/hooks/init/) de WordPress.
> Es decir no llames desde su interior a este hook ni a ninguno que se ejecute con anterioridad como [plugins_loaded](https://developer.wordpress.org/reference/hooks/plugins_loaded/).
