<?php
/**
 * Check the configuration of our pages and blocks.
 *
 * @package Flat101\Security\Controller
 */

namespace Flat101\Security\Controller;

use Flat101\Security\Model\Entity\Page;
use Flat101\Security\Service\PagesService;
use Flat101\Security\Util\WpNotice;
use WP_Error;

use const Flat101\Security\NAME;
use const Flat101\Security\PREFIX;

/**
 * Pages Checker class.
 */
class PagesChecker {

	/**
	 * Option name which saves the last modification of pages config file.
	 *
	 * @var string
	 */
	private static $checked_time_option = PREFIX . '__pages_checked_time';


	/**
	 * If it finds any error it will show it to the user.
	 */
	public function check() {
		$service = PagesService::instance();

		// Check configuration only when config or this file change.
		$checked_time = filemtime( FLAT101_SECURITY_PAGES_CONFIG_FILE ) . '|' . filemtime( __FILE__ );
		if ( get_option( static::$checked_time_option, 0 ) !== $checked_time ) {

			// Get the pages array.
			$pages = $service->get_pages();

			$result = $this->check_pages( $pages );

			// If everything went well, update the option.
			if ( ! $result instanceof WP_Error ) {
				update_option( static::$checked_time_option, $checked_time );
			} else {

				// We have some error in our configuration.
				// Keeps only the first element of the array without blocks.
				$pages = array_slice( $pages, 0, 1 );
				if ( $pages ) {
					$pages[0]->empty_blocks();
				}

				$service->set_pages( $pages );

				// Show a notification to the user.
				add_action(
					'admin_notices',
					function () use ( $result ) {
						WpNotice::error( NAME . ' 🚀 ' . $result->get_error_message() );
					}
				);
			}
		}
	}


	/**
	 * Check the pages and blocks to avoid programming errors.
	 *
	 * @param Page[] $pages Pages.
	 * @return WP_Error|null
	 */
	private function check_pages( $pages ) {
		$checked_page_keys  = array();
		$checked_block_keys = array();

		foreach ( $pages as $page ) {

			// Check the page.
			$page_key = $page->get_key();

			if ( in_array( $page_key, $checked_page_keys, true ) ) {
				return new WP_Error(
					'page_not_unique_key',
					__( 'Repeated page with the same key', 'flat101-security' ) . " \"{$page_key}\""
				);
			}

			$checked_page_keys[] = $page_key;

			// Check the blocks.
			foreach ( $page->get_blocks() as $block ) {
				$block_key = $block->get_key();

				if ( in_array( $block_key, $checked_block_keys, true ) ) {
					return new WP_Error(
						'block_not_unique_key',
						__( 'Repeated block with the same key', 'flat101-security' ) . " \"{$block_key}\""
					);
				}

				$checked_block_keys[] = $block_key;
			}
		}

		return null;
	}


	/**
	 * This method should be called when the plugin is uninstalled.
	 */
	public static function uninstall() {

		// Clear up options.
		delete_option( static::$checked_time_option );
	}
}
