<?php

/**
 * Filter Access by IP and User-Agent with logging.
 *
 * @package Flat101\Security\Model\Block
 */

namespace Flat101\Security\Model\Block;

use Carbon_Fields\Field\Field;
use Flat101\Security\Model\Entity\Block;

/**
 * FilterAccessIpBlock class.
 */
class FilterAccessIpBlock extends Block {

	const OPTION_NAME = 'filter_access_ip_settings';
	const LOG_FILE = FLAT101_SECURITY_LOGS;
	const LOG_LIMIT = 5000; // Maximum number of log lines


	/**
	 * Block constructor.
	 */
	public function __construct() {
		parent::__construct(
			esc_html__('Filter Access IP', 'flat101-security'),
			self::OPTION_NAME,
			array(
				'enable_whitelist_ips' => false,
				'whitelist_ips' => '',
				'enable_blocked_ips' => false,
				'blocked_ips' => '',
				'enable_allowed_user_agents' => false,
				'allowed_user_agents' => '',
				'enable_denied_user_agents' => false,
				'denied_user_agents' => '',
			)
		);
	}

	/**
	* Get Fields.
	*/
	public function get_fields() {
		$user_ip = $this->getUserIpAddr(); // Retrieve the user's current IP address.

		return array(
			Field::make('html', 'user_ip_description')
			     ->set_html('<p>Your current IP address is: <strong>' . esc_html($user_ip) . '</strong></p>'),

			Field::make('checkbox', $this->get_option_name('enable_whitelist_ips'), __('Enable Whitelist IPs', 'flat101-security'))
			     ->set_help_text(__('Check this box to enable whitelist IP functionality.', 'flat101-security')),

			Field::make('textarea', $this->get_option_name('whitelist_ips'), __('Whitelist IP Addresses', 'flat101-security'))
			     ->set_conditional_logic(array(
				     array(
					     'field' => $this->get_option_name('enable_whitelist_ips'),
					     'value' => true,
				     ),
			     ))
			     ->set_help_text(__('Enter the IP addresses that should always be allowed access, one per line. Example: 192.168.1.1', 'flat101-security'))
			     ->set_attribute('placeholder', 'e.g., 192.168.1.1'),

			Field::make('checkbox', $this->get_option_name('enable_blocked_ips'), __('Enable Blocked IPs', 'flat101-security'))
			     ->set_help_text(__('Check this box to enable IP blocking functionality.', 'flat101-security')),

			Field::make('textarea', $this->get_option_name('blocked_ips'), __('Blocked IP Addresses', 'flat101-security'))
			     ->set_conditional_logic(array(
				     array(
					     'field' => $this->get_option_name('enable_blocked_ips'),
					     'value' => true,
				     ),
			     ))
			     ->set_help_text(__('Enter individual IP addresses or IP ranges that should be blocked from access, one per line. Example: 192.168.1.3', 'flat101-security'))
			     ->set_attribute('placeholder', 'e.g., 192.168.1.3'),

			Field::make('checkbox', $this->get_option_name('enable_allowed_user_agents'), __('Enable Allowed User Agents', 'flat101-security'))
			     ->set_help_text(__('Check this box to enable user agent allow functionality.', 'flat101-security')),

			Field::make('textarea', $this->get_option_name('allowed_user_agents'), __('Allowed User Agents', 'flat101-security'))
			     ->set_conditional_logic(array(
				     array(
					     'field' => $this->get_option_name('enable_allowed_user_agents'),
					     'value' => true,
				     ),
			     ))
			     ->set_help_text(__('Enter user agents that should be allowed access, one per line. Example: Mozilla/5.0', 'flat101-security'))
			     ->set_attribute('placeholder', 'e.g., Mozilla/5.0'),

			Field::make('checkbox', $this->get_option_name('enable_denied_user_agents'), __('Enable Denied User Agents', 'flat101-security'))
			     ->set_help_text(__('Check this box to enable user agent deny functionality.', 'flat101-security')),

			Field::make('textarea', $this->get_option_name('denied_user_agents'), __('Denied User Agents', 'flat101-security'))
			     ->set_conditional_logic(array(
				     array(
					     'field' => $this->get_option_name('enable_denied_user_agents'),
					     'value' => true,
				     ),
			     ))
			     ->set_help_text(__('Enter user agents that should be blocked from accessing the site, one per line. Example: BadBot', 'flat101-security'))
			     ->set_attribute('placeholder', 'e.g., BadBot'),

			Field::make('checkbox', $this->get_option_name('log_blocked_attempts'), __('Log Blocked Attempts', 'flat101-security'))
			     ->set_help_text(__('Enable this option to log all blocked access attempts.', 'flat101-security'))
			     ->set_default_value(true),

			Field::make('textarea', $this->get_option_name('blocked_access_log'), __('Blocked Access Log', 'flat101-security'))
			     ->set_help_text(__('This log contains the last ' . self::LOG_LIMIT . ' blocked access attempts.', 'flat101-security'))
			     ->set_attribute('readOnly', 'true') // Corrected attribute key
			     ->set_default_value($this->get_log_contents()),
		);
	}

	/**
	 * Get log Contents
	 */
	private function get_log_contents() {
		// Check if the log file exists, is readable, and is not a directory
		if (file_exists(self::LOG_FILE) && is_readable(self::LOG_FILE) && is_file(self::LOG_FILE)) {
			// Open the file in read mode
			$file = fopen(self::LOG_FILE, 'r');
			// Read the log contents
			$log_contents = fread($file, filesize(self::LOG_FILE));
			// Close the file handle
			fclose($file);
			// Return the log contents
			return $log_contents;
		}
		// Return an empty string if the file can't be read or if it's a directory
		return '';
	}



	/**
	 * Initialize the block, setting up actions and filters.
	 */
	public function init() {
		add_action( 'init', array( $this, 'run' ) );
	}

	/**
	 * Run the IP/User-Agent check on every page load.
	 */
	public function run() {

		$user_ip = $this->getUserIpAddr(); // Retrieve the user's current IP address.

		// Check for a specific query parameter and a predefined value to bypass the functionality.
		// You should replace 'bypass_key' and 'your_secret_value' with your actual parameter and secret.
		if (isset($_GET['bypass_key']) && $_GET['bypass_key'] === BYPASS_QUERY_SECRET_VALUE) {
			return; // Skip executing the rest of the run() functionality.
		}

		// Check if the user's IP is in the whitelist
		$whitelist_ips = preg_split('/\r\n|\r|\n/', $this->settings['whitelist_ips']);
		if (in_array($user_ip, $whitelist_ips)) {
			return; // Skip the blocking functionality if the IP is whitelisted.
		}

		if ( $this->settings['enable_denied_user_agents'] ) {
			if ( ! $this->check_ip() || ! $this->check_user_agent() ) {
				$this->log_access_attempt( "Access denied for {$user_ip} - {$_SERVER['HTTP_USER_AGENT']}" );
				header( "HTTP/1.1 403 Forbidden" );
				exit( 'Access Forbidden' );
			}
		}
	}

	/**
	 * Check if the IP is allowed or denied.
	 */
	private function check_ip() {

		// Only perform the check if the setting is enabled
		if (!$this->settings['enable_blocked_ips']) {
			return true; // if not enabled, default to allowing the IP
		}

		$ip = $this->getUserIpAddr();
		$whitelist_ips = explode("\n", str_replace("\r", "", $this->settings['whitelist_ips']));
		$blocked_ips = explode("\n", str_replace("\r", "", $this->settings['blocked_ips']));
		$denied_ips = explode("\n", str_replace("\r", "", $this->settings['denied_ips']));

		// If IP is in whitelist, allow
		if (in_array($ip, $whitelist_ips)) {
			return true;
		}

		// If IP is in denied list, block
		if (in_array($ip, $denied_ips)) {
			return false;
		}

		// If IP is in blocked list, block
		if (in_array($ip, $blocked_ips)) {
			return false;
		}

		// If no rules match, allow
		return true;
	}

	/**
	 * Check if the User-Agent is allowed or denied.
	 */
	private function check_user_agent() {

		// Only perform the check if the setting is enabled
		if (!$this->settings['enable_denied_user_agents']) {
			return true; // if not enabled, default to allowing the IP
		}

		$user_agent = $_SERVER['HTTP_USER_AGENT'];
		$allowed_user_agents = explode("\n", str_replace("\r", "", $this->settings['allowed_user_agents']));
		$denied_user_agents = explode("\n", str_replace("\r", "", $this->settings['denied_user_agents']));

		// If User-Agent is in denied list, block
		foreach ($denied_user_agents as $denied) {
			if (stripos($user_agent, $denied) !== false) {
				return false;
			}
		}

		// If User-Agent is in allowed list, allow
		foreach ($allowed_user_agents as $allowed) {
			if (stripos($user_agent, $allowed) !== false) {
				return true;
			}
		}

		// If no rules match, allow
		return true;
	}

	/**
	 * Check if an IP is within a range.
	 */
	private function ip_in_range( $ip, $range ) {
		if ( strpos( $range, '/' ) !== false ) {
			// $range is in IP/CIDR format
			list( $range, $netmask ) = explode( '/', $range, 2 );
			if ( filter_var( $range, FILTER_VALIDATE_IP, FILTER_FLAG_IPV4 ) ) {
				$ip_decimal       = ip2long( $ip );
				$range_decimal    = ip2long( $range );
				$wildcard_decimal = pow( 2, ( 32 - $netmask ) ) - 1;
				$netmask_decimal  = ~$wildcard_decimal;

				return ( ( $ip_decimal & $netmask_decimal ) == ( $range_decimal & $netmask_decimal ) );
			}
		}

		return false;
	}

	/**
	 * Log the denied access attempt.
	 */
	private function log_access_attempt( $message ) {
		$log_message = '[' . date( 'Y-m-d H:i:s' ) . '] ' . $message . PHP_EOL;

		// Check if log file is writable or can be created
		if ( is_writable( self::LOG_FILE ) || ! file_exists( self::LOG_FILE ) && is_writable( dirname( self::LOG_FILE ) ) ) {
			// Maintain log size
			$this->maintain_log_size();

			// Append new log message
			file_put_contents( self::LOG_FILE, $log_message, FILE_APPEND | LOCK_EX );
		}
	}

	/**
	 * Helper method to maintain the log file size.
	 */
	private function maintain_log_size() {
		if ( file_exists( self::LOG_FILE ) ) {
			$lines = file( self::LOG_FILE, FILE_IGNORE_NEW_LINES | FILE_SKIP_EMPTY_LINES );
			if ( count( $lines ) > self::LOG_LIMIT ) {
				// Truncate the file to the last 5000 lines
				file_put_contents( self::LOG_FILE, implode( PHP_EOL, array_slice( $lines, - self::LOG_LIMIT ) ) . PHP_EOL );
			}
		}
	}

	/**
	 * Get the current user's IP address considering possible headers.
	 *
	 * @return string The IP address of the user.
	 */
	private function getUserIpAddr(){
		$ip_address = '';
		if (!empty($_SERVER['HTTP_CLIENT_IP'])){
			$ip_address = $_SERVER['HTTP_CLIENT_IP']; // Get the shared IP Address
		}else if(!empty($_SERVER['HTTP_X_FORWARDED_FOR'])){
			//Check if the proxy is used for IP/IPs
			// Split if multiple IP addresses exist and get the last IP address
			if (strpos($_SERVER['HTTP_X_FORWARDED_FOR'], ',') !== false) {
				$multiple_ips = explode(",", $_SERVER['HTTP_X_FORWARDED_FOR']);
				$ip_address = trim(current($multiple_ips));
			}else{
				$ip_address = $_SERVER['HTTP_X_FORWARDED_FOR'];
			}
		}else if(!empty($_SERVER['HTTP_X_FORWARDED'])){
			$ip_address = $_SERVER['HTTP_X_FORWARDED'];
		}else if(!empty($_SERVER['HTTP_FORWARDED_FOR'])){
			$ip_address = $_SERVER['HTTP_FORWARDED_FOR'];
		}else if(!empty($_SERVER['HTTP_FORWARDED'])){
			$ip_address = $_SERVER['HTTP_FORWARDED'];
		}else{
			if (!empty($_SERVER['REMOTE_ADDR'])) {
				$ip_address = $_SERVER['REMOTE_ADDR'];
			} else {
				// Handle the case where REMOTE_ADDR is not set
				$ip_address = '0.0.0.0'; // You can assign a default value or handle it as per your application logic
			}

		}
		return $ip_address;
	}

}
