<?php
/**
 * Login related functionalities.
 *
 * @package Flat101\Security\Model\Block
 */

namespace Flat101\Security\Model\Block;

use Carbon_Fields\Field\Field;
use Flat101\Security\Model\Entity\Block;

/**
 * LoginOptimizations class.
 */
class LoginOptimizationsBlock extends Block {

	/**
	 * Block constructor.
	 */
	public function __construct() {
		parent::__construct(
			esc_html__( 'Login Optimizations', 'flat101-security' ),
			'login-optimizations',
			array(
				'remove_error_message' => false,
				'email_on_failed_login' => false,
				'disable_app_passwords' => false,
				'restrict_usernames' => false,
			)
		);
	}

	public function get_fields() {

		$forbidden_users_default = 'admin,webmaster,editor,Administrador,SuperAdmin,moderador,Super Administrador';

		return array(
			Field::make( 'html', 'login_description' )
			     ->set_html(
			     // Add your descriptive text here, with references if necessary.
				     '<p>Manage various login related settings to enhance the security and user experience of your WordPress site.</p>'
			     ),
			Field::make( 'checkbox', $this->get_option_name( 'remove_error_message' ), __( 'Remove login error message', 'flat101-security' ) )
			     ->set_default_value( $this->default_settings['remove_error_message'] )
			     ->set_help_text('Hide error messages on the login screen to prevent information disclosure to potential attackers.'),
			Field::make( 'checkbox', $this->get_option_name( 'email_on_failed_login' ), __( 'Email admin on multiple failed login attempts', 'flat101-security' ) )
			     ->set_default_value( $this->default_settings['email_on_failed_login'] )
			     ->set_help_text('Send an email notification to the admin email address if there are multiple failed login attempts from the same IP address.'),
			Field::make( 'checkbox', $this->get_option_name( 'disable_app_passwords' ), __( 'Disable Application Passwords', 'flat101-security' ) )
			     ->set_default_value( $this->default_settings['disable_app_passwords'] )
			     ->set_help_text('Disable the use of application passwords, adding an extra layer of security to your site.'),
			Field::make( 'checkbox', $this->get_option_name( 'restrict_usernames' ), __( 'Restrict certain usernames', 'flat101-security' ) )
			     ->set_default_value( $this->default_settings['restrict_usernames'] )
			     ->set_help_text('Restrict the use of certain usernames to prevent impersonation and misuse.'),
			Field::make('textarea', $this->get_option_name('forbidden_usernames'), __('Forbidden Usernames', 'flat101-security'))
			     ->set_default_value($forbidden_users_default)  // Default forbidden usernames, one per line
			     ->set_attribute('placeholder', __('Enter forbidden usernames, one per line', 'flat101-security'))
			     ->set_rows(4),
		);
	}

	public function run() {

		if ($this->settings['email_on_failed_login']) {
			add_action('wp_login_failed', array($this, 'track_failed_login'));
		}

		if ( $this->settings['remove_error_message'] ) {
			add_filter('login_errors', function($a) { return null; });
		}

		if ( $this->settings['disable_app_passwords'] ) {
			add_filter( 'wp_is_application_passwords_available', '__return_false' );
		}

		if ( $this->settings['restrict_usernames'] ) {
			add_filter('validate_username', array($this, 'sozot_validate_username'), 10, 2);
			add_filter('registration_errors', array($this, 'sozot_registration_errors'));
		}
	}

	// Additional functions for username validation and error handling
	public function sozot_validate_username($valid, $username) {
		if (!$valid || (is_user_logged_in() && current_user_can('create_users'))) {
			return $valid;
		}

		$forbidden_usernames_text = $this->settings['forbidden_usernames'];
		$forbidden = array_map('trim', explode(',', $forbidden_usernames_text));

		$pages = get_pages(array('post_status' => 'publish'));
		$forbidden = array_merge($forbidden, wp_list_pluck($pages, 'post_name'));

		$username = strtolower($username);
		$valid = $valid && !strpos($username, ' ') && !in_array($username, $forbidden) && strlen($username) >= 5;

		return $valid;
	}

	public function sozot_registration_errors($errors) {
		if (isset($errors->errors['invalid_username'])) {
			$errors->errors['invalid_username'][0] = __('ERROR: Nombre de usuario no válido, elige otro.', 'sozot');
		}
		return $errors;
	}

	public function track_failed_login($username) {
		$ip = $_SERVER['REMOTE_ADDR'];
		$transient_key = "failed_login_attempts_{$ip}";
		$failed_attempts = get_transient($transient_key);

		if ($failed_attempts === false) {
			// Transient doesn't exist, create it with a value of 1.
			set_transient($transient_key, 1, HOUR_IN_SECONDS);
		} else {
			$failed_attempts++;
			set_transient($transient_key, $failed_attempts, HOUR_IN_SECONDS);

			if ($failed_attempts >= 3) {
				$this->send_failed_login_email($ip, $failed_attempts);
				// Optionally reset the count after email is sent
				delete_transient($transient_key);
			}
		}
	}

	public function send_failed_login_email($ip, $failed_attempts) {
		$admin_email = get_option('admin_email');
		$site_url = get_site_url();
		$host_ip = gethostbyname(gethostname());
		$wp_version = get_bloginfo('version');

		$subject = "Security Alert: Multiple Failed Login Attempts from {$ip} on {$site_url}";

		$message = "Security Alert Details:\n";
		$message .= "Site URL: {$site_url}\n";
		$message .= "Host IP: {$host_ip}\n";
		$message .= "WordPress Version: {$wp_version}\n";
		$message .= "**********************************************\n";
		$message .= "Client IP with Failed Attempts: {$ip}\n";
		$message .= "Failed Login Attempts: {$failed_attempts}\n";

		wp_mail($admin_email, $subject, $message);
	}


}
