<?php
/**
 * Overview of Plugin Functionalities for Main Options Page.
 *
 * @package Flat101\Security\Model\Block
 */

namespace Flat101\Security\Model\Block;

use Carbon_Fields\Field\Field;

use Flat101\Security\Model\Entity\Block;
use Flat101\Security\Model\Entity\Page;
use Flat101\Security\Service\PagesService;

/**
 * OverviewMain class.
 */
class OverviewMain extends Block {

	/**
	 * @var array An array to store the blocks data.
	 */
	private $blocks_data = [];


	/**
	 * The page property.
	 *
	 * @var Page|null
	 */
	private $page = null;


	/**
	 * Block constructor.
	 */
	public function __construct(Page $page = null) {
		$this->page = $page;
		parent::__construct(
			esc_html__('Overview Main', 'flat101-security'),
			'overview_main',
			array(
				'',
				)
		);
	}


	/**
	 * Retrieve the configuration URL for a block.
	 *
	 * @param Block $block The block for which to get the config URL.
	 * @return string The URL to the block's configuration page.
	 */
	private function get_block_config_url($block) {

		return admin_url('admin.php?page=configure-block&block=' . $block->get_key());
	}



	/**
	 * Returns the fields of the block so that the user can modify the configuration from the form.
	 * Este método debe ser implementado de acuerdo a cómo quieres mostrar y gestionar los campos en la administración.
	 *
	 * @return array Array of fields
	 */
	public function get_fields() {

		$this->populate_blocks_data_and_get_options();

		if (empty($this->blocks_data)) {
			$this->populate_blocks_data(); // A new method that encapsulates the logic from `run()`
		}

		// Ensure the blocks data is populated
		if (empty($this->blocks_data)) {
			$this->run();
		}

		$fields = [];

		foreach ($this->blocks_data as $block) {
			if ($block->get_key() === 'overview_main') {
				continue;
			}
			$settings = $block->get_settings();
			$title = $block->get_title();
			$config_url = $this->get_block_config_url($block);

			$html_content = "<h2 style='font-weight: bold; font-size: 22px;color:#2271b1;'>➔ " . esc_html($title) . "</h2><ul>";

			foreach ($settings as $setting_key => $setting_value) {
				$formatted_key = ucwords(str_replace('_', ' ', $setting_key));
				if ($setting_value) {
					$status = "<strong style='color: darkgreen;'>Active</strong>";
				} else {
					$status = 'Inactive';
				}
				$html_content .= '<li>' . esc_html($formatted_key) . ': ' . $status . '</li>';
			}

			$html_content .= '</ul>';

			$fields[] = Field::make('html', 'overview_' . $block->get_key())
			                 ->set_html($html_content);
		}

		return $fields;
	}






	private function populate_blocks_data_and_get_options() {
		$service = PagesService::instance();
		$this->blocks_data = []; // Reset the blocks data
		$options_list = []; // To store the options values

		// Define the keys of the pages you want to include
		$allowed_page_keys = [
			'flat101_security_main_options_page',
			//'flat101_security_network_options_page'
		];

		foreach ($service->get_pages() as $page) {
			// Check if the page's key is one of the allowed keys
			if (!in_array($page->get_key(), $allowed_page_keys)) {
				continue; // Skip this page if it's not one of the allowed ones
			}

			foreach ($page->get_blocks() as $block) {
				// Load the settings for the block, assuming it's an associative array of setting keys
				foreach ($block->settings as $key => $value) {
					// Update the settings array of the block
					$option_name = '_'. $block->get_option_name($key);
					$block->settings[$key] = get_option($option_name);

					// Add the option name and value to the list
					$options_list[$option_name] = $block->settings[$key];
				}
				$this->blocks_data[] = $block; // Store the updated block data
			}

		}

		// Now $options_list contains all the Carbon Fields options for the blocks
		return $options_list;
	}



	/**
	 * Execute the logic of this block.
	 * Este método debe contener la lógica principal de lo que hace tu bloque.
	 */
	public function run() {

	}



}
