<?php
/**
 * Security Headers.
 *
 * @package Flat101\Security\Model\Block
 */

namespace Flat101\Security\Model\Block;

use Carbon_Fields\Field\Field;
use Flat101\Security\Model\Entity\Block;

/**
 * Security Headers class.
 */
class SecurityHeadersBlock extends Block {

	const OPTION_NAME = 'security_headers_settings';

	/**
	 * Block constructor.
	 */
	public function __construct() {
		parent::__construct(
			esc_html__('Security Headers', 'flat101-security'),
			self::OPTION_NAME,
			array(
				//'disabled' => true,
				'enable_csp' => true,
				'csp_rules' => true,
				'x_frame_options' => true,
				'enable_hsts' => true,
				'enable_mime_type_sniffing' => true,
				'enable_xss_protection' => true,
			)
		);
	}


	/**
	 * Get Fields.
	 */
	public function get_fields() {

		//$default_csp = "default-src\n    'self'\n    ;\nfont-src\n    'self'\n    ;\nimg-src\n    'self'\n    ;\nstyle-src\n    'self'\n    ;\nscript-src\n    'self'\n    ;\nframe-src\n    'self'\n    ;\nconnect-src\n    'self'\n    ;";

		$default_csp ="default-src 'self';\n style-src 'self' 'unsafe-inline';\n img-src 'self' http://*.gravatar.com data:;\nscript-src 'self' 'unsafe-inline' 'unsafe-eval';\n font-src 'self' data:;";

		return array(
			Field::make('checkbox', $this->get_option_name('enable_csp'), __('Enable Content Security Policy', 'flat101-security'))
			     ->set_help_text(
				     __(
					     'The Content Security Policy (CSP) functionality provides a method to specify the sources of content that are allowed to be loaded by your WordPress site. By defining a policy, you can mitigate the risks associated with Cross-Site Scripting (XSS), Clickjacking, and other code injection attacks. It is advised to test the policy in a non-production environment to ensure that it doesn’t interfere with the functionality of your site. For more information, refer to the <a href="https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/Content-Security-Policy" target="_blank">MDN Web Docs on Content Security Policy</a>.',
					     'flat101-security'
				     )
			     )
			     ->set_default_value( $this->default_settings['enable_csp'] ),
			Field::make('textarea', $this->get_option_name('csp_rules'), __('Content Security Policy Rules', 'flat101-security'))
			     ->set_default_value($default_csp),
			Field::make('select', $this->get_option_name('x_frame_options'), __('X-Frame-Options', 'flat101-security'))
			     ->set_help_text(
				     __(
					     'The X-Frame-Options header functionality allows you to control whether your site can be embedded within frames on other websites. Setting this to "DENY" will prevent all framing, while "SAMEORIGIN" allows framing only by the same site. This can help protect against Clickjacking attacks. For more details, refer to the <a href="https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/X-Frame-Options" target="_blank">MDN Web Docs on X-Frame-Options</a>.',
					     'flat101-security'
				     )
			     )
			     ->add_options(array(
				     'deny' => 'DENY',
				     'sameorigin' => 'SAMEORIGIN'
			     )),
			Field::make('checkbox', $this->get_option_name('enable_hsts'), __('Enable HSTS Header', 'flat101-security'))
			     ->set_help_text(
				     __(
					     'The HTTP Strict Transport Security (HSTS) header enforces the use of HTTPS to communicate with your site. Enabling this option will add the HSTS header to your site’s responses, enhancing your site’s security by ensuring that all communications are encrypted.',
					     'flat101-security'
				     )
			     )
			     ->set_default_value( $this->default_settings['enable_hsts'] ),

			Field::make('checkbox', $this->get_option_name('enable_mime_type_sniffing'), __('Enable MIME Type Sniffing Protection', 'flat101-security'))
			     ->set_help_text(
				     __(
					     'MIME type sniffing protection prevents browsers from interpreting files as a different MIME type. This helps to reduce the risk of drive-by-download attacks.',
					     'flat101-security'
				     )
			     )
			     ->set_default_value( $this->default_settings['enable_mime_type_sniffing'] ),

			Field::make('checkbox', $this->get_option_name('enable_xss_protection'), __('Enable XSS Protection', 'flat101-security'))
			     ->set_help_text(
				     __(
					     'Cross-Site Scripting (XSS) protection helps to stop malicious scripts from executing in your users’ browsers. Enabling this option will add the X-XSS-Protection header to your site’s responses.',
					     'flat101-security'
				     )
			     )
			     ->set_default_value( $this->default_settings['enable_xss_protection'] ),

		);
	}


	/**
	 * Save the security headers settings.
	 */
	public function save_headers() {
		if ($_SERVER['REQUEST_METHOD'] === 'POST') {
			$settings = array(
				'enable_csp' => isset($_POST[$this->get_option_name('enable_csp')]) ? $_POST[$this->get_option_name('enable_csp')] : '',
				'csp_rules' => isset($_POST[$this->get_option_name('csp_rules')]) ? $_POST[$this->get_option_name('csp_rules')] : '',
				'x_frame_options' => isset($_POST[$this->get_option_name('x_frame_options')]) ? $_POST[$this->get_option_name('x_frame_options')] : '',
			);

			// Validation logic (simplified)
			if (empty($settings['csp_rules'])) {
				add_settings_error(
					'flat101-security',
					esc_attr('settings_updated'),
					__('CSP rules cannot be empty.', 'flat101-security'),
					'error'
				);
			} else {
				// Save the settings as a single option in the WordPress database
				update_option(self::OPTION_NAME, $settings);
			}
		}
	}

	/**
	 * Add the action hook to save the headers and set the security headers.
	 */
	public function init() {
		add_action('admin_init', array($this, 'save_headers'));
		add_action('init', array($this, 'run'));
	}

	/**
	 * Set the security headers based on saved settings.
	 */
	public function run() {
		if (!headers_sent()) {
			$this->set_content_security_policy_header();
			$this->set_x_frame_options_header();
			$this->set_hsts_header();
			$this->set_mime_type_sniffing_header();
			$this->set_xss_protection_header();
		} else {
			error_log('Headers already sent, unable to set security headers.');
		}
	}

	private function set_content_security_policy_header() {
		if ( $this->settings['enable_csp'] === true && !empty($this->settings['csp_rules'])) {
			// Remove newline characters and extra spaces from the csp_rules string
			$csp_rules = str_replace(array("\r", "\n"), '', $this->settings['csp_rules']);
			$csp_rules = preg_replace('/\s+/', ' ', $csp_rules);
			header('Content-Security-Policy: ' . $csp_rules);
		}
	}

	private function set_x_frame_options_header() {
		if (!empty($this->settings['x_frame_options'])) {
			header('X-Frame-Options: ' . $this->settings['x_frame_options']);
		}
	}

	private function set_hsts_header() {
		if ($this->settings['enable_hsts']) {
			if (0 === strpos($_SERVER['HTTP_HOST'], 'localhost')) {
				header('Strict-Transport-Security: max-age=0; includeSubdomains; preload');
			} else {
				header('Strict-Transport-Security: max-age=31536000; includeSubDomains; preload');
			}
		}
	}

	private function set_mime_type_sniffing_header() {
		if ($this->settings['enable_mime_type_sniffing']) {
			header('X-Content-Type-Options: nosniff');
		}
	}

	private function set_xss_protection_header() {
		if ($this->settings['enable_xss_protection']) {
			header('X-XSS-Protection: 1; mode=block');
		}
	}
}


/*
 				echo "<pre>";
				var_dump($this->settings['enable_csp']);
				var_dump($this->settings['csp_rules']);
				//print_r($this->settings);
				echo "</pre>";
				die("4444");
 */
?>
