<?php
/**
 * Debugging utilities. They should not be used in a production environment.
 *
 * @package Flat101\Security\Util
 */

namespace Flat101\Security\Util;

use WP_Error;

use const Flat101\Security\PREFIX;

/**
 * Useful class for debugging.
 */
class Debug {

	const CHRONO_START = PREFIX . '__chrono_start';

	/**
	 * Write to the log.
	 *
	 * @param string $message Text to write in the log.
	 */
	private static function error_log( $message ) {
    // phpcs:disable WordPress.PHP.DevelopmentFunctions.error_log_error_log
		if ( isset( $_ENV['DEBUG_LOG'] ) ) {
			// [15-Jun-2021 13:22:17 UTC] xxx
			$txt = '[' . gmdate( 'd-M-Y H:i:s \U\T\C' ) . '] ' . $message . "\n";
			error_log( $txt, 3, sanitize_text_field( $_ENV['DEBUG_LOG'] ) );
		} else {
			error_log( $message );
		}
    // phpcs:enable WordPress.PHP.DevelopmentFunctions.error_log_error_log
	}

	/**
	 * Designed to write any type of data to the log.
	 *
	 * If WP_DEBUG is on write to:
	 *     /wp-content/debug.log
	 *
	 * @param mixed[] ...$items As many parameters as you need of whatever type.
	 */
	public static function debug( ...$items ) {
		if ( defined( 'WP_DEBUG' ) && ( true === WP_DEBUG ) ) {
			foreach ( $items as $item ) {
				if ( $item instanceof WP_Error ) {
					$item = array(
						'errors' => $item->errors,
						'data'   => $item->get_all_error_data(),
					);
				}

				static::error_log(
					wp_json_encode( $item, JSON_UNESCAPED_UNICODE | JSON_UNESCAPED_SLASHES | JSON_PRETTY_PRINT )
				);
			}
		}
	}

	/**
	 * Write the text of the message with optional data.
	 *
	 * @param string $message Text to write in the log.
	 * @param mixed  ...$data Dataset related to this message. As many parameters as you need.
	 */
	public static function log( $message, ...$data ) {
		if ( defined( 'WP_DEBUG' ) && ( true === WP_DEBUG ) ) {
			static::error_log( $message );

			if ( ! empty( $data ) ) {
				static::debug( ...$data );
			}
		}
	}

	/**
	 * Write to the log the last query to the database.
	 */
	public static function last_query() {
		/**
		 * WP Database Object.
		 *
		 * @var \wpdb $wpdb
		 */
		global $wpdb;

		$last_query = preg_replace( '/\s+/', ' ', $wpdb->last_query );

		static::debug( $last_query );
	}

	/**
	 * Write to the log all the queries to the database until now.
	 */
	public static function queries() {
		/**
		 * WP Database Object.
		 *
		 * @var \wpdb $wpdb
		 */
		global $wpdb;

		static::debug( $wpdb->queries );
	}


	/**
	 * Take the current instant as the start of the chrono.
	 */
	public static function chrono_start() {
    // phpcs:ignore WordPress.NamingConventions.PrefixAllGlobals.NonPrefixedVariableFound
		$GLOBALS[ self::CHRONO_START ] = microtime( true );
	}

	/**
	 * Writes to the log the time elapsed since the last time 'chrono_start()' was called.
	 * If 'chrono_start()' has not been called previously it will not write anything.
	 */
	public static function chrono_end() {
		$flat_chrono_end = microtime( true );

		if ( isset( $GLOBALS[ self::CHRONO_START ] ) ) {
			static::log( 'Time: ' . round( ( $flat_chrono_end - $GLOBALS[ self::CHRONO_START ] ) * 1000, 0 ) . ' ms' );
		}
	}


	/**
	 * Echo any type of data to the screen (only if WP_DEBUG is on).
	 *
	 * @param mixed $data Whatever you want.
	 */
	public static function pre( $data ) {
		if ( defined( 'WP_DEBUG' ) && ( true === WP_DEBUG ) ) {
			if ( $data instanceof WP_Error ) {
				$data = array(
					'errors' => $data->errors,
					'data'   => $data->get_all_error_data(),
				);
			}

			echo "<pre>\n";
			echo wp_json_encode( $data, JSON_UNESCAPED_UNICODE | JSON_UNESCAPED_SLASHES | JSON_PRETTY_PRINT ) . "\n";
			echo "</pre>\n";
		}
	}


	/**
	 * Write to log the current request.
	 */
	public static function request() {
		if (
		defined( 'WP_DEBUG' ) &&
		( true === WP_DEBUG ) &&
		! empty( $_SERVER['REQUEST_URI'] ) &&
    // phpcs:ignore WordPress.Security.NonceVerification.Missing
		! ( isset( $_POST['action'] ) && 'heartbeat' === $_POST['action'] )
		) {
			static::log( '-----------------------------------------------------------------' );

			$info = array(
				isset( $_SERVER['REQUEST_METHOD'] ) ? sanitize_text_field( wp_unslash( $_SERVER['REQUEST_METHOD'] ) ) : null,
				isset( $_SERVER['REQUEST_URI'] ) ? esc_url_raw( wp_unslash( $_SERVER['REQUEST_URI'] ) ) : null,
			);
			static::log( implode( ' ', $info ) );

      // phpcs:disable WordPress.Security.NonceVerification.Recommended
			if ( isset( $_REQUEST['action'] ) ) {
				static::log( 'ACTION: ' . sanitize_text_field( wp_unslash( $_REQUEST['action'] ) ) );
			}
      // phpcs:enable WordPress.Security.NonceVerification.Recommended

			// phpcs:disable WordPress.Security.NonceVerification.Missing
			if ( count( $_POST ) > 0 ) {
				// phpcs:ignore WordPress.PHP.DevelopmentFunctions.error_log_print_r
				static::log( 'POST: ' . print_r( $_POST, true ) );
			}
			// phpcs:enable WordPress.Security.NonceVerification.Missing
		}
	}
}
